<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN"
   "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
<html xmlns="http://www.w3.org/1999/xhtml"
      xmlns:meld="http://www.plope.com/software/meld3">
<head>
</head>
<body>

<div meld:id="content">

  <h1>Recent Errors</h1>
  
  <ul>
    
    <li meld:id="error_li">
      
      <b meld:id="error_time"/> <a href="#" meld:id="error_url">Description</a>
      
    </li>
    
  </ul>
</div>

</body>
</html>
